@@ -1,16 +1,21 @@
+from __future__ import absolute_import
 '''
 Differential equations for Brian models.
 '''
-import collections
+try:
+    from collections.abc import Mapping, Hashable
+except ImportError:  # Python 2
+    from collections import Mapping, Hashable
 import keyword
 import re
 import string
 
-import sympy
-from brian2.utils.stringtools import get_identifiers
 from pyparsing import (Group, ZeroOrMore, OneOrMore, Optional, Word, CharsNotIn,
                        Combine, Suppress, restOfLine, LineEnd, ParseException)
+import sympy
+from past.builtins import basestring
 
+from brian2.utils.stringtools import get_identifiers
 from brian2.core.namespace import (DEFAULT_FUNCTIONS,
                                    DEFAULT_CONSTANTS,
                                    DEFAULT_UNITS)
@@ -237,7 +242,7 @@ def dimensions_and_type_from_string(unit_string):
     global _base_units
     if _base_units_with_alternatives is None:
         base_units_for_dims = {}
-        for unit_name, unit in reversed(list(DEFAULT_UNITS.items())):
+        for unit_name, unit in reversed(DEFAULT_UNITS.items()):
             if float(unit) == 1.0 and repr(unit)[-1] not in ['2', '3']:
                 if unit.dim in base_units_for_dims:
                     if unit_name not in base_units_for_dims[unit.dim]:
@@ -246,7 +251,8 @@ def dimensions_and_type_from_string(unit_string):
                     base_units_for_dims[unit.dim] = [repr(unit)]
                     if unit_name != repr(unit):
                         base_units_for_dims[unit.dim].append(unit_name)
-        alternatives = sorted([tuple(values) for values in base_units_for_dims.values()])
+        alternatives = sorted([tuple(values)
+                               for values in base_units_for_dims.values()])
         _base_units = dict([(v, DEFAULT_UNITS[v])
                             for values in alternatives for v in values])
         # Create a string that lists all allowed base units
@@ -349,7 +355,7 @@ def parse_string_equations(eqns):
                             ' ' * (p_exc.column - 1) + '^\n' + str(p_exc))
     for eq in parsed:
         eq_type = eq.getName()
-        eq_content = dict(list(eq.items()))
+        eq_content = dict(eq.items())
         # Check for reserved keywords
         identifier = eq_content['identifier']
 
@@ -380,7 +386,7 @@ def parse_string_equations(eqns):
     return equations
 
 
-class SingleEquation(collections.Hashable, CacheKey):
+class SingleEquation(Hashable, CacheKey):
     '''
     Class for internal use, encapsulates a single equation or parameter.
 
@@ -521,7 +527,7 @@ class SingleEquation(collections.Hashable, CacheKey):
         return '$' + sympy.latex(self) + '$'
 
 
-class Equations(collections.Hashable, collections.Mapping):
+class Equations(Hashable, Mapping):
     """
     Container that stores equations from which models can be created.
     
@@ -552,7 +558,7 @@ class Equations(collections.Hashable, collections.Mapping):
     """
 
     def __init__(self, eqns, **kwds):
-        if isinstance(eqns, str):
+        if isinstance(eqns, basestring):
             self._equations = parse_string_equations(eqns)
             # Do a basic check for the identifiers
             self.check_identifiers()
@@ -600,7 +606,7 @@ class Equations(collections.Hashable, collections.Mapping):
             # Replace the name of a model variable (works only for strings)
             if eq.varname in replacements:
                 new_varname = replacements[eq.varname]
-                if not isinstance(new_varname, str):
+                if not isinstance(new_varname, basestring):
                     raise ValueError(('Cannot replace model variable "%s" '
                                       'with a value') % eq.varname)
                 if new_varname in self or new_varname in new_equations:
@@ -619,7 +625,7 @@ class Equations(collections.Hashable, collections.Mapping):
                 new_code = eq.expr.code
                 for to_replace, replacement in replacements.items():
                     if to_replace in eq.identifiers:
-                        if isinstance(replacement, str):
+                        if isinstance(replacement, basestring):
                             # replace the name with another name
                             new_code = re.sub('\\b' + to_replace + '\\b',
                                               replacement, new_code)
@@ -660,7 +666,7 @@ class Equations(collections.Hashable, collections.Mapping):
         return self._equations[key]
 
     def __add__(self, other_eqns):
-        if isinstance(other_eqns, str):
+        if isinstance(other_eqns, basestring):
             other_eqns = parse_string_equations(other_eqns)
         elif not isinstance(other_eqns, Equations):
             return NotImplemented
@@ -668,7 +674,7 @@ class Equations(collections.Hashable, collections.Mapping):
         return Equations(list(self.values()) + list(other_eqns.values()))
 
     def __hash__(self):
-        return hash(frozenset(iter(self._equations.items())))
+        return hash(frozenset(self._equations.items()))
 
     #: A set of functions that are used to check identifiers (class attribute).
     #: Functions can be registered with the static method
@@ -794,9 +800,7 @@ class Equations(collections.Hashable, collections.Mapping):
             time), ``'multiplicative'`` (at least one of the noise factors
             depends on other state variables and/or time).
         '''
-        
-        # TODO: Add debug output
-        
+
         if not self.is_stochastic:
             return None
         
@@ -821,8 +825,9 @@ class Equations(collections.Hashable, collections.Mapping):
     ############################################################################
 
     # Lists of equations or (variable, expression tuples)
-    ordered = property(lambda self: sorted(iter(self._equations.values()),
-                                           key=lambda key: key.update_order),
+    ordered = property(lambda self: sorted(self._equations.values(),
+                                           key=lambda key: (key.update_order,
+                                                            key.varname)),
                                            doc='A list of all equations, sorted '
                                            'according to the order in which they should '
                                            'be updated')
